@@ -5,7 +5,7 @@ import sys
 
 from fabric import state
 from fabric.utils import abort, warn, error
-from fabric.network import to_dict, normalize_to_string
+from fabric.network import to_dict, normalize_to_string, disconnect_all
 from fabric.context_managers import settings
 from fabric.job_queue import JobQueue
 from fabric.task_utils import crawl, merge, parse_kwargs
@@ -240,15 +240,16 @@ def execute(task, *args, **kwargs):
     kwarg1='value')`` will (once per host) invoke ``mytask('arg1',
     kwarg1='value')``.
 
-    This function returns a dictionary mapping host strings to the given task's
-    return value for that host's execution run. For example, ``execute(foo,
-    hosts=['a', 'b'])`` might return ``{'a': None, 'b': 'bar'}`` if ``foo``
-    returned nothing on host `a` but returned ``'bar'`` on host `b`.
+    :returns:
+        a dictionary mapping host strings to the given task's return value for
+        that host's execution run. For example, ``execute(foo, hosts=['a',
+        'b'])`` might return ``{'a': None, 'b': 'bar'}`` if ``foo`` returned
+        nothing on host `a` but returned ``'bar'`` on host `b`.
 
-    In situations where a task execution fails for a given host but overall
-    progress does not abort (such as when :ref:`env.skip_bad_hosts
-    <skip-bad-hosts>` is True) the return value for that host will be the error
-    object or message.
+        In situations where a task execution fails for a given host but overall
+        progress does not abort (such as when :ref:`env.skip_bad_hosts
+        <skip-bad-hosts>` is True) the return value for that host will be the
+        error object or message.
 
     .. seealso::
         :ref:`The execute usage docs <execute>`, for an expanded explanation
@@ -325,6 +326,10 @@ def execute(task, *args, **kwargs):
                 else:
                     raise
 
+            # If requested, clear out connections here and not just at the end.
+            if state.env.eagerly_disconnect:
+                disconnect_all()
+
         # If running in parallel, block until job queue is emptied
         if jobs:
             err = "One or more hosts failed while executing task '%s'" % (
